/*
 * This file is part of SubLink.
 * Copyright (C) 2009  Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

"use strict";

var EXPORTED_SYMBOLS = ["enableButton", "disableButton",
                        "addToToolbar", "buttonCmd", "AnimateButton",
                        "buttonClick", "buttonTooltip", "loadImage"];

Components.utils.import("resource://sublink-migreme-modules/util.js");

var m_iconCanvas = null;

function enableButton(doc) {
  addCommandToDom(doc);
  addButtonToDom(doc);
  //AnimateButton.done(doc.defaultView); // m_iconCanvas ainda eh null
}


function disableButton(doc) {
  var cmdset = doc.getElementById("cmd_sublink-migreme-copy-url").parentNode;
  var win = cmdset.parentNode;
  win.removeChild(cmdset);

  var p = doc.getElementById("navigator-toolbox").palette;
  var button2 = p.children.namedItem("sublink-migreme-button");
  if (button2 !== null) {
    p.removeChild(button2);
  }

  var button = doc.getElementById("sublink-migreme-button");
  if (button !== null) {
    button.parentNode.removeChild(button);
  }
}


function addButtonToDom(doc) {
  var button = doc.createElement("toolbarbutton");
  var mid = "sublink-migreme-button";
  button.setAttribute("id", mid);
  button.setAttribute("label", "SubLink migre.me");
  button.setAttribute("command", "cmd_sublink-migreme-copy-url");
  button.setAttribute("class", "toolbarbutton-1 chromeclass-toolbar-additional");

  // 16x16 transp image
  button.style.listStyleImage = "url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAACXBIWXMAAAsSAAALEgHS3X78AAAAIUlEQVQ4EWP8//8/AyWAiRLNIL2jBoyGwWg6gOSigc8LAGOaAx0s99CyAAAAAElFTkSuQmCC)";
  button.style.MozBinding = "url(chrome://sublink-migreme/content/bindings.xml#toolbarbuttonSmall)";

  doc.getElementById("navigator-toolbox").palette.appendChild(button);

  var toolbarElem = findToolbar(doc, mid);
  if (toolbarElem !== null) {
    var beforeNode = findBeforeNode(doc, toolbarElem, mid);
    toolbarElem.insertItem(mid, beforeNode, null, false);
  }
}


function addCommandToDom(doc) {
  var cmdButton = doc.createElement("command");

  cmdButton.setAttribute("id", "cmd_sublink-migreme-copy-url");
  cmdButton.setAttribute("oncommand", "var jsm={};Cu.import('resource://sublink-migreme-modules/button.js',jsm);jsm.buttonCmd(event);");
  cmdButton.setAttribute("onclick", "var jsm={};Cu.import('resource://sublink-migreme-modules/button.js',jsm);jsm.buttonClick(event);");
  cmdButton.setAttribute("tooltiptext", "*");

  cmdButton.setAttribute("label", util.getText("button.create.label", "migre.me"));
  cmdButton.hidden = false;


  var d1 = doc.createElement("commandset");
  d1.appendChild(cmdButton);
  doc.documentElement.appendChild(d1);
}


function findBeforeNode(doc, bar, mid) {
  var all = bar.getAttribute("currentset").split(",");
  var idxButton = all.indexOf(mid);
  if (idxButton > -1) {
    var beforeNode;
    for (var idx = idxButton + 1, len = all.length; idx < len; idx++) {
      beforeNode = doc.getElementById(all[idx]);
      if (beforeNode !== null) {
        return beforeNode;
      }
    }
  }
  return null;
}


function findToolbar(doc, id) {
  var toolbars = doc.getElementsByTagName("toolbar");
  for (var idx = toolbars.length - 1; idx > -1; idx--) {
    var bar = toolbars[idx];
    var all = bar.getAttribute("currentset").split(",");
    if (all.indexOf(id) > -1) {
      return bar;
    }
  }
  return null;
}








function addToToolbar(doc) {
  var newId = "sublink-migreme-button";
  var toolbar = doc.getElementById("nav-bar");
  if (toolbar.currentSet.split(",").indexOf(newId) > -1) {
    return;
  }

  var newSet = toolbar.currentSet + "," + newId;
  console.log("addToToolbar newSet", toolbar.currentSet, newSet);
  toolbar.setAttribute("currentset", newSet);
  toolbar.currentSet = newSet;
  doc.persist("nav-bar", "currentset");
  doc.defaultView.BrowserToolboxCustomizeDone(true);
}




/*

 * Installs the toolbar button with the given ID into the given
 * toolbar, if it is not already present in the document.
 *
 * @param {string} toolbarId The ID of the toolbar to install to.
 * @param {string} id The ID of the button to install.
 * @param {string} afterId The ID of the element to insert after. @optional

function installButton(toolbarId, id, afterId) {
    if (!document.getElementById(id)) {
        var toolbar = document.getElementById(toolbarId);

        var before = toolbar.firstChild;
        if (afterId) {
            let elem = document.getElementById(afterId);
            if (elem && elem.parentNode == toolbar) {
                before = elem.nextElementSibling;
            }
        }

        toolbar.insertItem(id, before);
        toolbar.setAttribute("currentset", toolbar.currentSet);
        document.persist(toolbar.id, "currentset");

        if (toolbarId == "addon-bar")
            toolbar.collapsed = false;
    }
}

if (firstRun) {
    installButton("nav-bar", "my-extension-navbar-button", "urlbar");
    installButton("addon-bar", "my-extension-addon-bar-button");
}
*/







function buttonClick(evt) {
  if (evt.button !== 1 || evt.detail !== 1) {
    return;
  }

  // middle click
  var button = evt.target;
  var win = button.ownerDocument.defaultView;
  util.openPage("http://migre.me/", "tab", win);
}


function buttonCmd(evt) {
  var command = evt.target;

  var win = command.ownerDocument.defaultView;
  var browser = win.getBrowser().selectedBrowser;
  var uri = util.stringToUri(browser.contentDocument.location.href);

  Components.utils.import("resource://sublink-migreme-modules/redir-storage.js");
  var redir = redirStorage.getSavedRedir(uri);
  if (redir) {
    util.toClipboard(redir);
    AnimateButton.done(win);
    return;
  }

  function onComplete() {
    AnimateButton.waitStop(win);
  }

  function onSuccess(info) {
    AnimateButton.done(win);
    util.toClipboard(info.shortUri.spec);
  }

  function onError(errData) {
    Components.utils.import("resource://sublink-migreme-modules/err.js");
    notificationError(errData, browser);
  }

  AnimateButton.waitStart(win);
  util.toClipboard("");
  Components.utils.import("resource://sublink-migreme-modules/util2.js");
  convertUrl(uri, onSuccess, onError, onComplete);
}


function buttonTooltip(tooltip) {
  var win = tooltip.ownerDocument.defaultView;
  var uri = win.getBrowser().selectedBrowser.contentDocument.documentURIObject;
  var text;

  Components.utils.import("resource://sublink-migreme-modules/redir-storage.js");
  var redir = redirStorage.getSavedRedir(uri);
  if (redir) {
    text = util.getText("button.create.saved.tooltip", redir);
  } else {
    text = util.getText("button.create.new.tooltip", "migre.me");
  }
  tooltip.setAttribute("label", text);
}




function loadImage(button) {
  var size = 16;
  var doc = button.ownerDocument;

  if (m_iconCanvas) {
    canvasToButton(m_iconCanvas, button);
    return;
  }

  var cnv = createHtmlElement("canvas", doc);
  cnv.width = size;
  cnv.height = size;
  var ctx = cnv.getContext("2d");

  var favImg = createHtmlElement("img", doc);

  favImg.onload = function(evt) {
    //ctx.globalAlpha = 0.8;
    ctx.drawImage(favImg, 0, 0);
    m_iconCanvas = cnv; //ctx.getImageData(0, 0, size, size);
    canvasToButton(cnv, button);
  };

  favImg.onerror = function(evt) {
    var ns = {};
    Components.utils.import("resource://gre/modules/AddonManager.jsm", ns);
    ns.AddonManager.getAddonByID("migre.me@hultmann", function(addon) {
      var appImg = createHtmlElement("img", doc);
      appImg.onload = function(evt) {
        var imgScale = size / 32;
        ctx.scale(imgScale, imgScale);
        ctx.drawImage(appImg, 0, 0);
        m_iconCanvas = cnv;
        canvasToButton(cnv, button);
      }
      appImg.src = addon.iconURL;
    });
  };

  favImg.src = "http://migre.me/favicon.ico";
}


function createHtmlElement(tagName, xulDoc) {
  return xulDoc.createElementNS("http://www.w3.org/1999/xhtml", tagName);
}


function canvasToButton(canvas, button) {
  button.style.listStyleImage = "url(" + canvas.toDataURL("image/png") + ")";
}


var AnimateButton = {
  _getButton: function(win) {
    return win.document.getElementById("sublink-migreme-button");
  },

  waitStart: function(win) {
    var button = this._getButton(win);
    if (button.anim) {
      button.anim.finish();
    }
    button.anim = new WorkingButton(win); // TODO (button) em vez de win
    button.anim.wait();
  },

  waitStop: function(win) {
    var button = this._getButton(win);
    if (button.anim) {
      button.anim.finish();
    }
  },

  done: function(win) {
    var button = this._getButton(win);
    if (!button.anim) {
      button.anim = new WorkingButton(win);
    }
    button.anim.success();
  },

  remove: function(win) {
    delete this._getButton(win).anim;
  },

  easing: {
    success: function(d) {
      return (1 - Math.cos(Math.PI * d)) / 2;
    },
    waiting: function(d) {
      return -d;
    }
  }
};


function WorkingButton(win) {
  this._win = Components.utils.getWeakReference(win);

  var cnv = createHtmlElement("canvas", win.document);
  cnv.width = 16;
  cnv.height = 16;
  this._cnv = cnv;
}


WorkingButton.prototype = {

  finish: function() {
    var win = this._win.get(); // BUG null ao clicar durante animacao
    AnimateButton.remove(win);
    if (win !== null) {
      this._renderFrame(0, "waiting"); // reset icon
    }
    this._win = null;
    this._cnv = null;
  },

  wait: function() {
    this._isSuccess = false;
    this._start = this._win.get().mozAnimationStartTime;
    this._waitCore(this._start);
  },


  success: function() {
    this._isSuccess = true;
    this._start = this._win.get().mozAnimationStartTime;
    this._successCore(this._start);
  },

  _waitCore: function(now) {
    if (this._isSuccess || (this._cnv === null)) {
      return; // this is a remaining requestAnimationFrame
    }
    var duration = 9000;
    var rotation = (now - this._start) / duration;
    if (rotation > 1) {
      rotation -= 1;
    }
    this._renderFrame(rotation, "waiting");
    var me = this;
    this._win.get().mozRequestAnimationFrame(function(now) {
      me._waitCore.call(me, now);
    });
  },

  _successCore: function(now) {
    var duration = 400;
    var rotation = (now - this._start) / duration;
    if (rotation > 1) {
      this.finish();
      return;
    }

    this._renderFrame(rotation, "success");
    var me = this;
    this._win.get().mozRequestAnimationFrame(function(now) {
      me._successCore.call(me, now);
    });
  },

  _renderFrame: function(rotation, easeFunction) {
    var offsetY = 1;
    var size = 16;
    var size2 = size / 2;
    var ctx = this._cnv.getContext("2d");
    ctx.save();
    ctx.clearRect(0, 0, size, size);
    //ctx.putImageData(m_iconCanvas, 0, 0);
    ctx.translate(size2, size2 + offsetY);
    var rotation2 = AnimateButton.easing[easeFunction](rotation);
    ctx.rotate(2 * Math.PI * rotation2);
    ctx.drawImage(m_iconCanvas, -size2, -size2 - offsetY);
    ctx.restore();

    var button = this._win.get().document.getElementById("sublink-migreme-button");
    canvasToButton(this._cnv, button);
  }
};
